@@ -29,7 +29,11 @@ from dagster._core.definitions.freshness_policy import FreshnessPolicy
 from dagster._core.definitions.multi_dimensional_partitions import (
     MultiPartitionsSubset,
 )
-from dagster._core.definitions.partition import DefaultPartitionsSubset, PartitionsSubset
+from dagster._core.definitions.partition import (
+    DefaultPartitionsSubset,
+    PartitionsDefinition,
+    PartitionsSubset,
+)
 from dagster._core.definitions.time_window_partitions import (
     TimeWindowPartitionsDefinition,
     TimeWindowPartitionsSubset,
@@ -38,14 +42,17 @@ from dagster._core.events import ASSET_EVENTS
 from dagster._core.host_representation.external import ExternalRepository
 from dagster._core.host_representation.external_data import ExternalAssetNode
 from dagster._core.host_representation.repository_location import RepositoryLocation
+from dagster._core.instance import DynamicPartitionsStore
 from dagster._core.storage.partition_status_cache import (
-    get_and_update_asset_status_cache_values,
+    CACHEABLE_PARTITION_TYPES,
+    get_and_update_asset_status_cache_value,
     get_materialized_multipartitions,
     get_validated_partition_keys,
 )
 from dagster._utils.caching_instance_queryer import CachingInstanceQueryer
 
 from dagster_graphql.implementation.loader import (
+    CachingDynamicPartitionsLoader,
     CrossRepoAssetDependedByLoader,
     StaleStatusLoader,
 )
@@ -172,6 +179,8 @@ def get_asset_nodes_by_asset_key(
         asset_graph=lambda: ExternalAssetGraph.from_workspace(graphene_info.context),
     )
 
+    dynamic_partitions_loader = CachingDynamicPartitionsLoader(graphene_info.context.instance)
+
     asset_nodes_by_asset_key: Dict[AssetKey, GrapheneAssetNode] = {}
     for repo_loc, repo, external_asset_node in asset_node_iter(graphene_info):
         preexisting_node = asset_nodes_by_asset_key.get(external_asset_node.asset_key)
@@ -182,6 +191,7 @@ def get_asset_nodes_by_asset_key(
                 external_asset_node,
                 depended_by_loader=depended_by_loader,
                 stale_status_loader=stale_status_loader,
+                dynamic_partitions_loader=dynamic_partitions_loader,
             )
 
     return asset_nodes_by_asset_key
@@ -314,23 +324,25 @@ def get_unique_asset_id(
 def get_materialized_partitions_subset(
     instance: DagsterInstance,
     asset_key: AssetKey,
-    asset_graph: ExternalAssetGraph,
+    dynamic_partitions_loader: DynamicPartitionsStore,
+    partitions_def: Optional[PartitionsDefinition] = None,
 ) -> Optional[PartitionsSubset]:
     """
     Returns the materialization status for each partition key. The materialization status
     is a boolean indicating whether the partition has been materialized: True if materialized,
     False if not.
     """
-    partitions_def = asset_graph.get_partitions_def(asset_key)
     if not partitions_def:
         return None
 
-    if instance.can_cache_asset_status_data():
+    if instance.can_cache_asset_status_data() and isinstance(
+        partitions_def, CACHEABLE_PARTITION_TYPES
+    ):
         # When the "cached_status_data" column exists in storage, update the column to contain
         # the latest partition status values
-        updated_cache_value = get_and_update_asset_status_cache_values(
-            instance, asset_graph, asset_key
-        ).get(asset_key)
+        updated_cache_value = get_and_update_asset_status_cache_value(
+            instance, asset_key, partitions_def
+        )
         materialized_subset = (
             updated_cache_value.deserialize_materialized_partition_subsets(partitions_def)
             if updated_cache_value
@@ -354,7 +366,9 @@ def get_materialized_partitions_subset(
                 if count > 0
             ]
 
-        validated_keys = get_validated_partition_keys(partitions_def, set(materialized_keys))
+        validated_keys = get_validated_partition_keys(
+            dynamic_partitions_loader, partitions_def, set(materialized_keys)
+        )
 
         return (
             partitions_def.empty_subset().with_partition_keys(validated_keys)
@@ -364,6 +378,7 @@ def get_materialized_partitions_subset(
 
 
 def build_materialized_partitions(
+    dynamic_partitions_store: DynamicPartitionsStore,
     materialized_partitions_subset: Optional[PartitionsSubset],
 ) -> Union["GrapheneTimePartitions", "GrapheneDefaultPartitions", "GrapheneMultiPartitions",]:
     from ..schema.pipelines.pipeline import (
@@ -393,17 +408,22 @@ def build_materialized_partitions(
             ]
         )
     elif isinstance(materialized_partitions_subset, MultiPartitionsSubset):  # Multidimensional
-        return get_2d_run_length_encoded_materialized_partitions(materialized_partitions_subset)
+        return get_2d_run_length_encoded_materialized_partitions(
+            dynamic_partitions_store, materialized_partitions_subset
+        )
     elif isinstance(materialized_partitions_subset, DefaultPartitionsSubset):
         return GrapheneDefaultPartitions(
             materializedPartitions=materialized_partitions_subset.get_partition_keys(),
-            unmaterializedPartitions=materialized_partitions_subset.get_partition_keys_not_in_subset(),
+            unmaterializedPartitions=materialized_partitions_subset.get_partition_keys_not_in_subset(
+                dynamic_partitions_store=dynamic_partitions_store
+            ),
         )
     else:
         check.failed("Should not reach this point")
 
 
 def get_2d_run_length_encoded_materialized_partitions(
+    dynamic_partitions_store: DynamicPartitionsStore,
     partitions_subset: PartitionsSubset,
 ) -> "GrapheneMultiPartitions":
     from ..schema.pipelines.pipeline import (
@@ -432,16 +452,23 @@ def get_2d_run_length_encoded_materialized_partitions(
 
     materialized_2d_ranges = []
 
-    dim1_keys = primary_dim.partitions_def.get_partition_keys()
+    dim1_keys = primary_dim.partitions_def.get_partition_keys(
+        dynamic_partitions_store=dynamic_partitions_store
+    )
     unevaluated_idx = 0
     range_start_idx = 0  # pointer to first dim1 partition with same dim2 materialization status
 
+    if len(dim1_keys) == 0 or len(secondary_dim.partitions_def.get_partition_keys()) == 0:
+        return GrapheneMultiPartitions(ranges=[], primaryDimensionName=primary_dim.name)
+
     while unevaluated_idx <= len(dim1_keys):
         if (
             unevaluated_idx == len(dim1_keys)
             or dim2_partition_subset_by_dim1[dim1_keys[unevaluated_idx]]
             != dim2_partition_subset_by_dim1[dim1_keys[range_start_idx]]
         ):
+            # Add new multipartition range if we've reached the end of the dim1 keys or if the
+            # second dimension subset is different than the previous dim1 key
             if len(dim2_partition_subset_by_dim1[dim1_keys[range_start_idx]]) > 0:
                 # Do not add to materialized_2d_ranges if the dim2 partition subset is empty
                 start_key = dim1_keys[range_start_idx]
@@ -465,7 +492,8 @@ def get_2d_run_length_encoded_materialized_partitions(
                         primaryDimStartTime=start_time,
                         primaryDimEndTime=end_time,
                         secondaryDim=build_materialized_partitions(
-                            dim2_partition_subset_by_dim1[start_key]
+                            dynamic_partitions_store,
+                            dim2_partition_subset_by_dim1[start_key],
                         ),
                     )
                 )
