@@ -112,6 +112,7 @@ def get_partition_config(graphene_info, repository_handle, partition_set_name, p
         repository_handle,
         partition_set_name,
         partition_name,
+        graphene_info.context.instance,
     )
 
     return GraphenePartitionRunConfig(yaml=dump_run_config_yaml(result.run_config))
@@ -127,7 +128,7 @@ def get_partition_tags(graphene_info, repository_handle, partition_set_name, par
     check.str_param(partition_name, "partition_name")
 
     result = graphene_info.context.get_external_partition_tags(
-        repository_handle, partition_set_name, partition_name
+        repository_handle, partition_set_name, partition_name, graphene_info.context.instance
     )
 
     return GraphenePartitionTags(
@@ -281,7 +282,7 @@ def get_partition_set_partition_runs(graphene_info: ResolveInfo, partition_set):
 
     by_partition = {}
     for record in run_records:
-        partition_name = record.pipeline_run.tags.get(PARTITION_NAME_TAG)
+        partition_name = record.dagster_run.tags.get(PARTITION_NAME_TAG)
         if not partition_name or partition_name in by_partition:
             # all_partition_set_runs is in descending order by creation time, we should ignore
             # runs for the same partition if we've already considered the partition
