@@ -16,10 +16,12 @@ from dagster._core.host_representation.external_data import (
     ExternalAssetDependency,
     ExternalAssetNode,
 )
-from dagster._core.scheduler.instigation import InstigatorType
+from dagster._core.instance import DynamicPartitionsStore
+from dagster._core.scheduler.instigation import InstigatorState, InstigatorType
 from dagster._core.storage.pipeline_run import JobBucket, RunRecord, RunsFilter, TagBucket
 from dagster._core.storage.tags import REPOSITORY_LABEL_TAG, SCHEDULE_NAME_TAG, SENSOR_NAME_TAG
 from dagster._core.workspace.context import WorkspaceRequestContext
+from dagster._utils.cached_method import cached_method
 
 
 class RepositoryDataType(Enum):
@@ -96,7 +98,7 @@ class RepositoryScopedBatchLoader:
                         )
                     )
             for record in records:
-                fetched[record.pipeline_run.pipeline_name].append(record)
+                fetched[record.dagster_run.pipeline_name].append(record)
 
         elif data_type == RepositoryDataType.SCHEDULE_RUNS:
             schedule_names = [
@@ -132,7 +134,7 @@ class RepositoryScopedBatchLoader:
                         )
                     )
             for record in records:
-                tag: str = check.not_none(record.pipeline_run.tags.get(SCHEDULE_NAME_TAG))
+                tag: str = check.not_none(record.dagster_run.tags.get(SCHEDULE_NAME_TAG))
                 fetched[tag].append(record)
 
         elif data_type == RepositoryDataType.SENSOR_RUNS:
@@ -167,7 +169,7 @@ class RepositoryScopedBatchLoader:
                         )
                     )
             for record in records:
-                tag = check.not_none(record.pipeline_run.tags.get(SENSOR_NAME_TAG))
+                tag = check.not_none(record.dagster_run.tags.get(SENSOR_NAME_TAG))
                 fetched[tag].append(record)
 
         elif data_type == RepositoryDataType.SCHEDULE_STATES:
@@ -240,7 +242,7 @@ class RepositoryScopedBatchLoader:
         check.invariant(self._repository.has_external_sensor(sensor_name))
         return self._get(RepositoryDataType.SENSOR_RUNS, sensor_name, limit)
 
-    def get_schedule_state(self, schedule_name: str) -> Optional[Sequence[Any]]:
+    def get_schedule_state(self, schedule_name: str) -> Optional[InstigatorState]:
         check.invariant(self._repository.has_external_schedule(schedule_name))
         states = self._get(RepositoryDataType.SCHEDULE_STATES, schedule_name, 1)
         return states[0] if states else None
@@ -293,7 +295,7 @@ class BatchRunLoader:
     def _fetch(self) -> None:
         records = self._instance.get_run_records(RunsFilter(run_ids=list(self._run_ids)))
         for record in records:
-            self._records[record.pipeline_run.run_id] = record
+            self._records[record.dagster_run.run_id] = record
 
 
 class BatchMaterializationLoader:
@@ -329,6 +331,20 @@ class BatchMaterializationLoader:
         }
 
 
+class CachingDynamicPartitionsLoader(DynamicPartitionsStore):
+    """
+    A batch loader that caches the partition keys for a given dynamic partitions definition,
+    to avoid repeated calls to the database for the same partitions definition.
+    """
+
+    def __init__(self, instance: DagsterInstance):
+        self._instance = instance
+
+    @cached_method
+    def get_dynamic_partitions(self, partitions_def_name: str) -> Sequence[str]:
+        return self._instance.get_dynamic_partitions(partitions_def_name)
+
+
 class CrossRepoAssetDependedByLoader:
     """
     A batch loader that computes cross-repository asset dependencies. Locates source assets
