@@ -1,4 +1,4 @@
-from typing import TYPE_CHECKING, List, Optional, Sequence, Union, cast
+from typing import TYPE_CHECKING, Any, List, Optional, Sequence, Union, cast
 
 import graphene
 from dagster import (
@@ -8,13 +8,16 @@ from dagster import (
 from dagster._core.definitions.external_asset_graph import ExternalAssetGraph
 from dagster._core.definitions.logical_version import (
     NULL_LOGICAL_VERSION,
+    StaleStatus,
 )
+from dagster._core.errors import DagsterInvariantViolationError
 from dagster._core.event_api import EventRecordsFilter
 from dagster._core.events import DagsterEventType
 from dagster._core.host_representation import ExternalRepository, RepositoryLocation
 from dagster._core.host_representation.external import ExternalPipeline
 from dagster._core.host_representation.external_data import (
     ExternalAssetNode,
+    ExternalDynamicPartitionsDefinitionData,
     ExternalMultiPartitionsDefinitionData,
     ExternalPartitionsDefinitionData,
     ExternalStaticPartitionsDefinitionData,
@@ -47,6 +50,7 @@ from ..implementation.fetch_assets import (
 )
 from ..implementation.loader import (
     BatchMaterializationLoader,
+    CachingDynamicPartitionsLoader,
     CrossRepoAssetDependedByLoader,
     StaleStatusLoader,
 )
@@ -76,6 +80,17 @@ from .util import ResolveInfo, non_null_list
 if TYPE_CHECKING:
     from .external import GrapheneRepository
 
+GrapheneAssetStaleStatus = graphene.Enum.from_enum(StaleStatus, name="StaleStatus")
+
+
+class GrapheneAssetStaleStatusCause(graphene.ObjectType):
+    status = graphene.NonNull(GrapheneAssetStaleStatus)
+    key = graphene.NonNull(GrapheneAssetKey)
+    reason = graphene.NonNull(graphene.String)
+
+    class Meta:
+        name = "StaleStatusCause"
+
 
 class GrapheneAssetDependency(graphene.ObjectType):
     class Meta:
@@ -216,6 +231,8 @@ class GrapheneAssetNode(graphene.ObjectType):
     projectedLogicalVersion = graphene.String()
     repository = graphene.NonNull(lambda: external.GrapheneRepository)
     required_resources = non_null_list(GrapheneResourceRequirement)
+    staleStatus = graphene.Field(GrapheneAssetStaleStatus)
+    staleStatusCauses = non_null_list(GrapheneAssetStaleStatusCause)
     type = graphene.Field(GrapheneDagsterType)
 
     class Meta:
@@ -229,6 +246,7 @@ class GrapheneAssetNode(graphene.ObjectType):
         materialization_loader: Optional[BatchMaterializationLoader] = None,
         depended_by_loader: Optional[CrossRepoAssetDependedByLoader] = None,
         stale_status_loader: Optional[StaleStatusLoader] = None,
+        dynamic_partitions_loader: Optional[CachingDynamicPartitionsLoader] = None,
     ):
         from ..implementation.fetch_assets import get_unique_asset_id
 
@@ -254,6 +272,9 @@ class GrapheneAssetNode(graphene.ObjectType):
             "stale_status_loader",
             StaleStatusLoader,
         )
+        self._dynamic_partitions_loader = check.opt_inst_param(
+            dynamic_partitions_loader, "dynamic_partitions_loader", CachingDynamicPartitionsLoader
+        )
         self._external_pipeline = None  # lazily loaded
         self._node_definition_snap = None  # lazily loaded
 
@@ -311,7 +332,7 @@ class GrapheneAssetNode(graphene.ObjectType):
             )
             self._node_definition_snap = self.get_external_pipeline().get_node_def_snap(node_key)
         # weird mypy bug causes mistyped _node_definition_snap
-        return check.not_none(self._node_definition_snap)  # type: ignore
+        return check.not_none(self._node_definition_snap)
 
     def get_partition_keys(
         self,
@@ -327,6 +348,10 @@ class GrapheneAssetNode(graphene.ObjectType):
         )
         check.opt_int_param(start_idx, "start_idx")
         check.opt_int_param(end_idx, "end_idx")
+
+        if not self._dynamic_partitions_loader:
+            check.failed("dynamic_partitions_loader must be provided to get partition keys")
+
         partitions_def_data = (
             self._external_asset_node.partitions_def_data
             if not partitions_def_data
@@ -352,8 +377,18 @@ class GrapheneAssetNode(graphene.ObjectType):
                 else:
                     return [
                         partition.name
-                        for partition in partitions_def_data.get_partitions_definition().get_partitions()
+                        for partition in partitions_def_data.get_partitions_definition().get_partitions(
+                            dynamic_partitions_store=self._dynamic_partitions_loader
+                        )
                     ]
+            elif isinstance(partitions_def_data, ExternalDynamicPartitionsDefinitionData):
+                return self._dynamic_partitions_loader.get_dynamic_partitions(
+                    partitions_def_name=partitions_def_data.name
+                )
+            else:
+                raise DagsterInvariantViolationError(
+                    f"Unsupported partition definition type {partitions_def_data}"
+                )
         return []
 
     def is_multipartitioned(self) -> bool:
@@ -393,10 +428,11 @@ class GrapheneAssetNode(graphene.ObjectType):
         return self._external_asset_node.is_source
 
     def resolve_assetMaterializationUsedData(
-        self, graphene_info: ResolveInfo, **kwargs
+        self,
+        graphene_info: ResolveInfo,
+        timestampMillis: str,
     ) -> Sequence[GrapheneMaterializationUpstreamDataVersion]:
-        timestamp_millis: Optional[str] = kwargs.get("timestampMillis")
-        if not timestamp_millis:
+        if not timestampMillis:
             return []
 
         instance = graphene_info.context.instance
@@ -409,8 +445,8 @@ class GrapheneAssetNode(graphene.ObjectType):
         event_records = instance.get_event_records(
             EventRecordsFilter(
                 event_type=DagsterEventType.ASSET_MATERIALIZATION,
-                before_timestamp=int(timestamp_millis) / 1000.0 + 1,
-                after_timestamp=int(timestamp_millis) / 1000.0 - 1,
+                before_timestamp=int(timestampMillis) / 1000.0 + 1,
+                after_timestamp=int(timestampMillis) / 1000.0 - 1,
                 asset_key=asset_key,
             ),
             limit=1,
@@ -438,9 +474,12 @@ class GrapheneAssetNode(graphene.ObjectType):
         ]
 
     def resolve_assetMaterializations(
-        self, graphene_info: ResolveInfo, **kwargs
+        self,
+        graphene_info: ResolveInfo,
+        partitions: Optional[Sequence[Optional[str]]] = None,
+        beforeTimestampMillis: Optional[str] = None,
+        limit: Optional[int] = None,
     ) -> Sequence[GrapheneMaterializationEvent]:
-        beforeTimestampMillis: Optional[str] = kwargs.get("beforeTimestampMillis")
         try:
             before_timestamp = (
                 int(beforeTimestampMillis) / 1000.0 if beforeTimestampMillis else None
@@ -448,8 +487,6 @@ class GrapheneAssetNode(graphene.ObjectType):
         except ValueError:
             before_timestamp = None
 
-        limit = kwargs.get("limit")
-        partitions = kwargs.get("partitions")
         if (
             self._latest_materialization_loader
             and limit == 1
@@ -479,18 +516,18 @@ class GrapheneAssetNode(graphene.ObjectType):
         ]
 
     def resolve_assetObservations(
-        self, graphene_info: ResolveInfo, **kwargs
+        self,
+        graphene_info: ResolveInfo,
+        partitions: Optional[Sequence[Optional[str]]] = None,
+        beforeTimestampMillis: Optional[str] = None,
+        limit: Optional[int] = None,
     ) -> Sequence[GrapheneObservationEvent]:
-        beforeTimestampMillis: Optional[str] = kwargs.get("beforeTimestampMillis")
         try:
             before_timestamp = (
                 int(beforeTimestampMillis) / 1000.0 if beforeTimestampMillis else None
             )
         except ValueError:
             before_timestamp = None
-        limit = kwargs.get("limit")
-        partitions = kwargs.get("partitions")
-
         return [
             GrapheneObservationEvent(event=event)
             for event in get_asset_observations(
@@ -519,7 +556,23 @@ class GrapheneAssetNode(graphene.ObjectType):
     def resolve_computeKind(self, _graphene_info: ResolveInfo) -> Optional[str]:
         return self._external_asset_node.compute_kind
 
-    def resolve_currentLogicalVersion(self, _graphene_info: ResolveInfo) -> Optional[str]:
+    def resolve_staleStatus(self, graphene_info: ResolveInfo) -> Any:  # (GrapheneAssetStaleStatus)
+        return self.stale_status_loader.get_status(self._external_asset_node.asset_key)
+
+    def resolve_staleStatusCauses(
+        self, graphene_info: ResolveInfo
+    ) -> Sequence[GrapheneAssetStaleStatusCause]:
+        causes = self.stale_status_loader.get_status_causes(self._external_asset_node.asset_key)
+        return [
+            GrapheneAssetStaleStatusCause(
+                cause.status,
+                GrapheneAssetKey(path=cause.key.path),
+                cause.reason,
+            )
+            for cause in causes
+        ]
+
+    def resolve_currentLogicalVersion(self, graphene_info: ResolveInfo) -> Optional[str]:
         version = self.stale_status_loader.get_current_logical_version(
             self._external_asset_node.asset_key
         )
@@ -661,13 +714,15 @@ class GrapheneAssetNode(graphene.ObjectType):
         return self._external_asset_node.is_observable
 
     def resolve_latestMaterializationByPartition(
-        self, graphene_info: ResolveInfo, **kwargs
+        self,
+        graphene_info: ResolveInfo,
+        partitions: Optional[Sequence[Optional[str]]] = None,
     ) -> Sequence[Optional[GrapheneMaterializationEvent]]:
         get_partition = (
             lambda event: event.dagster_event.step_materialization_data.materialization.partition
         )
 
-        partitions = kwargs.get("partitions") or self.get_partition_keys()
+        partitions = partitions or self.get_partition_keys()
 
         events_for_partitions = get_asset_materializations(
             graphene_info,
@@ -697,21 +752,40 @@ class GrapheneAssetNode(graphene.ObjectType):
     def resolve_materializedPartitions(
         self, graphene_info: ResolveInfo
     ) -> Union[GrapheneDefaultPartitions, GrapheneTimePartitions, GrapheneMultiPartitions]:
-        asset_graph = ExternalAssetGraph.from_external_repository(self._external_repository)
         asset_key = self._external_asset_node.asset_key
+
+        if not self._dynamic_partitions_loader:
+            check.failed("dynamic_partitions_loader must be provided to get partition keys")
+
         materialized_partition_subset = get_materialized_partitions_subset(
-            graphene_info.context.instance, asset_key, asset_graph
+            graphene_info.context.instance,
+            asset_key,
+            self._dynamic_partitions_loader,
+            self._external_asset_node.partitions_def_data.get_partitions_definition()
+            if self._external_asset_node.partitions_def_data
+            else None,
         )
 
-        return build_materialized_partitions(materialized_partition_subset)
+        return build_materialized_partitions(
+            self._dynamic_partitions_loader,
+            materialized_partition_subset,
+        )
 
     def resolve_partitionStats(self, graphene_info) -> Optional[GraphenePartitionStats]:
         partitions_def_data = self._external_asset_node.partitions_def_data
         if partitions_def_data:
             asset_key = self._external_asset_node.asset_key
-            asset_graph = ExternalAssetGraph.from_external_repository(self._external_repository)
+
+            if not self._dynamic_partitions_loader:
+                check.failed("dynamic_partitions_loader must be provided to get partition keys")
+
             materialized_partition_subset = get_materialized_partitions_subset(
-                graphene_info.context.instance, asset_key, asset_graph
+                graphene_info.context.instance,
+                asset_key,
+                self._dynamic_partitions_loader,
+                self._external_asset_node.partitions_def_data.get_partitions_definition()
+                if self._external_asset_node.partitions_def_data
+                else None,
             )
 
             if materialized_partition_subset is None:
@@ -719,7 +793,9 @@ class GrapheneAssetNode(graphene.ObjectType):
 
             return GraphenePartitionStats(
                 numMaterialized=len(materialized_partition_subset),
-                numPartitions=partitions_def_data.get_partitions_definition().get_num_partitions(),
+                numPartitions=partitions_def_data.get_partitions_definition().get_num_partitions(
+                    dynamic_partitions_store=self._dynamic_partitions_loader
+                ),
             )
         else:
             return None
@@ -766,7 +842,9 @@ class GrapheneAssetNode(graphene.ObjectType):
                 GrapheneDimensionPartitionKeys(
                     name=dimension.name,
                     partition_keys=self.get_partition_keys(
-                        dimension.external_partitions_def_data, start_idx, end_idx
+                        dimension.external_partitions_def_data,
+                        start_idx,
+                        end_idx,
                     ),
                 )
                 for dimension in cast(
