@@ -95,6 +95,7 @@ class GraphenePartitionBackfill(graphene.ObjectType):
     backfillId = graphene.NonNull(graphene.String)
     status = graphene.NonNull(GrapheneBulkActionStatus)
     partitionNames = non_null_list(graphene.String)
+    isValidSerialization = graphene.NonNull(graphene.Boolean)
     numPartitions = graphene.NonNull(graphene.Int)
     numCancelable = graphene.NonNull(graphene.Int)
     fromFailure = graphene.NonNull(graphene.Boolean)
@@ -185,7 +186,7 @@ class GraphenePartitionBackfill(graphene.ObjectType):
         from .pipelines.pipeline import GrapheneRun
 
         records = self._get_records(graphene_info)
-        return [GrapheneRun(record) for record in records if not record.pipeline_run.is_finished]
+        return [GrapheneRun(record) for record in records if not record.dagster_run.is_finished]
 
     def resolve_runs(self, graphene_info: ResolveInfo):
         from .pipelines.pipeline import GrapheneRun
@@ -193,6 +194,9 @@ class GraphenePartitionBackfill(graphene.ObjectType):
         records = self._get_records(graphene_info)
         return [GrapheneRun(record) for record in records]
 
+    def resolve_isValidSerialization(self, _graphene_info: ResolveInfo):
+        return self._backfill_job.is_valid_serialization(_graphene_info.context)
+
     def resolve_partitionNames(self, _graphene_info: ResolveInfo):
         return self._backfill_job.get_partition_names(_graphene_info.context)
 
