@@ -29,6 +29,7 @@ from dagster._core.workspace.workspace import (
 
 from dagster_graphql.implementation.fetch_solids import get_solid, get_solids
 from dagster_graphql.implementation.loader import (
+    CachingDynamicPartitionsLoader,
     RepositoryScopedBatchLoader,
     StaleStatusLoader,
 )
@@ -39,6 +40,7 @@ from .partition_sets import GraphenePartitionSet
 from .permissions import GraphenePermission
 from .pipelines.pipeline import GrapheneJob, GraphenePipeline
 from .repository_origin import GrapheneRepositoryMetadata, GrapheneRepositoryOrigin
+from .resources import GrapheneResourceDetails
 from .schedules import GrapheneSchedule
 from .sensors import GrapheneSensor
 from .used_solid import GrapheneUsedSolid
@@ -47,6 +49,14 @@ from .util import ResolveInfo, non_null_list
 GrapheneLocationStateChangeEventType = graphene.Enum.from_enum(LocationStateChangeEventType)
 
 
+class GrapheneDagsterLibraryVersion(graphene.ObjectType):
+    name = graphene.NonNull(graphene.String)
+    version = graphene.NonNull(graphene.String)
+
+    class Meta:
+        name = "DagsterLibraryVersion"
+
+
 class GrapheneRepositoryLocationLoadStatus(graphene.Enum):
     LOADING = "LOADING"
     LOADED = "LOADED"
@@ -72,11 +82,12 @@ class GrapheneRepositoryLocation(graphene.ObjectType):
     environment_path = graphene.String()
     repositories = non_null_list(lambda: GrapheneRepository)
     server_id = graphene.String()
+    dagsterLibraryVersions = graphene.List(graphene.NonNull(GrapheneDagsterLibraryVersion))
 
     class Meta:
         name = "RepositoryLocation"
 
-    def __init__(self, location):
+    def __init__(self, location: RepositoryLocation):
         self._location = check.inst_param(location, "location", RepositoryLocation)
         environment_path = (
             location.origin.loadable_target_origin.executable_path
@@ -97,7 +108,7 @@ class GrapheneRepositoryLocation(graphene.ObjectType):
             server_id=server_id,
         )
 
-    def resolve_id(self, _):
+    def resolve_id(self, _) -> str:
         return self.name
 
     def resolve_repositories(self, graphene_info: ResolveInfo):
@@ -106,6 +117,13 @@ class GrapheneRepositoryLocation(graphene.ObjectType):
             for repository in self._location.get_repositories().values()
         ]
 
+    def resolve_dagsterLibraryVersions(self, _: ResolveInfo):
+        libs = self._location.get_dagster_library_versions()
+        if libs is None:
+            return None
+
+        return [GrapheneDagsterLibraryVersion(name, ver) for name, ver in libs.items()]
+
 
 class GrapheneRepositoryLocationOrLoadError(graphene.Union):
     class Meta:
@@ -213,6 +231,7 @@ class GrapheneRepository(graphene.ObjectType):
     assetNodes = non_null_list(GrapheneAssetNode)
     displayMetadata = non_null_list(GrapheneRepositoryMetadata)
     assetGroups = non_null_list(GrapheneAssetGroup)
+    allTopLevelResourceDetails = non_null_list(GrapheneResourceDetails)
 
     class Meta:
         name = "Repository"
@@ -233,6 +252,7 @@ class GrapheneRepository(graphene.ObjectType):
             instance=instance,
             asset_graph=lambda: ExternalAssetGraph.from_external_repository(repository),
         )
+        self._dynamic_partitions_loader = CachingDynamicPartitionsLoader(instance)
         super().__init__(name=repository.name)
 
     def resolve_id(self, _graphene_info: ResolveInfo):
@@ -315,6 +335,7 @@ class GrapheneRepository(graphene.ObjectType):
                 self._repository,
                 external_asset_node,
                 stale_status_loader=self._stale_status_loader,
+                dynamic_partitions_loader=self._dynamic_partitions_loader,
             )
             for external_asset_node in self._repository.get_external_asset_nodes()
         ]
@@ -334,6 +355,14 @@ class GrapheneRepository(graphene.ObjectType):
             for group_name, external_nodes in groups.items()
         ]
 
+    def resolve_allTopLevelResourceDetails(self, _graphene_info):
+        return [
+            GrapheneResourceDetails(resource)
+            for resource in sorted(
+                self._repository.get_external_resources(), key=lambda resource: resource.name
+            )
+        ]
+
 
 class GrapheneRepositoryConnection(graphene.ObjectType):
     nodes = non_null_list(GrapheneRepository)
