@@ -1,12 +1,14 @@
 import sys
 import warnings
-from typing import Union
+from typing import Optional, Union
 
 import dagster._check as check
 import graphene
 import pendulum
 from dagster._core.definitions.run_request import RunRequest
 from dagster._core.definitions.schedule_definition import ScheduleExecutionData
+from dagster._core.definitions.selector import ScheduleSelector, SensorSelector
+from dagster._core.definitions.sensor_definition import SensorExecutionData
 from dagster._core.scheduler.instigation import (
     InstigatorState,
     InstigatorTick,
@@ -25,7 +27,15 @@ from ..implementation.fetch_instigators import get_tick_log_events
 from ..implementation.fetch_schedules import get_schedule_next_tick
 from ..implementation.fetch_sensors import get_sensor_next_tick
 from ..implementation.loader import RepositoryScopedBatchLoader
-from .errors import GrapheneError, GraphenePythonError
+from ..implementation.utils import UserFacingGraphQLError
+from .errors import (
+    GrapheneError,
+    GraphenePythonError,
+    GrapheneRepositoryLocationNotFound,
+    GrapheneRepositoryNotFoundError,
+    GrapheneScheduleNotFoundError,
+    GrapheneSensorNotFoundError,
+)
 from .logs.log_level import GrapheneLogLevel
 from .repository_origin import GrapheneRepositoryOrigin
 from .tags import GraphenePipelineTag
@@ -158,7 +168,7 @@ class GrapheneInstigationTick(graphene.ObjectType):
             return []
 
         records_by_id = {
-            record.pipeline_run.run_id: record
+            record.dagster_run.run_id: record
             for record in instance.get_run_records(RunsFilter(run_ids=run_ids))
         }
 
@@ -168,88 +178,125 @@ class GrapheneInstigationTick(graphene.ObjectType):
         return get_tick_log_events(graphene_info, self._tick)
 
 
-class GrapheneFutureInstigationTick(graphene.ObjectType):
-    timestamp = graphene.NonNull(graphene.Float)
+class GrapheneDryRunInstigationTick(graphene.ObjectType):
+    timestamp = graphene.Float()
     evaluationResult = graphene.Field(lambda: GrapheneTickEvaluation)
 
     class Meta:
-        name = "FutureInstigationTick"
+        name = "DryRunInstigationTick"
 
-    def __init__(self, state, timestamp):
-        self._state = check.inst_param(state, "state", InstigatorState)
-        self._timestamp = timestamp
+    def __init__(
+        self,
+        selector: Union[ScheduleSelector, SensorSelector],
+        timestamp: Optional[float],
+        cursor: Optional[str] = None,
+    ):
+        self._selector = check.inst_param(selector, "selector", (ScheduleSelector, SensorSelector))
+        self._cursor = cursor
         super().__init__(
-            timestamp=check.float_param(timestamp, "timestamp"),
+            timestamp=check.opt_float_param(timestamp, "timestamp"),
         )
 
     def resolve_evaluationResult(self, graphene_info: ResolveInfo):
-        if not self._state.is_running or self._state.instigator_type != InstigatorType.SCHEDULE:
-            return None
-
-        repository_origin = self._state.origin.external_repository_origin
-        if not graphene_info.context.has_repository_location(
-            repository_origin.repository_location_origin.location_name
-        ):
-            return None
+        if not graphene_info.context.has_repository_location(self._selector.location_name):
+            raise UserFacingGraphQLError(
+                GrapheneRepositoryLocationNotFound(location_name=self._selector.location_name)
+            )
 
         repository_location = graphene_info.context.get_repository_location(
-            repository_origin.repository_location_origin.location_name
+            self._selector.location_name
         )
-        if not repository_location.has_repository(repository_origin.repository_name):
-            return None
-
-        repository = repository_location.get_repository(repository_origin.repository_name)
-
-        if not repository.has_external_schedule(self._state.name):
-            return None
-
-        external_schedule = repository.get_external_schedule(self._state.name)
-        timezone_str = external_schedule.execution_timezone
-        if not timezone_str:
-            timezone_str = "UTC"
-
-        next_tick_datetime = next(external_schedule.execution_time_iterator(self._timestamp))
-        schedule_time = to_timezone(pendulum.instance(next_tick_datetime), timezone_str)
-        schedule_data: Union[ScheduleExecutionData, SerializableErrorInfo]
-        try:
-            schedule_data = repository_location.get_external_schedule_execution_data(
-                instance=graphene_info.context.instance,
-                repository_handle=repository.handle,
-                schedule_name=external_schedule.name,
-                scheduled_execution_time=schedule_time,
+        if not repository_location.has_repository(self._selector.repository_name):
+            raise UserFacingGraphQLError(
+                GrapheneRepositoryNotFoundError(
+                    repository_location_name=self._selector.location_name,
+                    repository_name=self._selector.repository_name,
+                )
             )
-        except Exception:
-            schedule_data = serializable_error_info_from_exc_info(sys.exc_info())
 
-        return GrapheneTickEvaluation(schedule_data)
+        repository = repository_location.get_repository(self._selector.repository_name)
+
+        if isinstance(self._selector, SensorSelector):
+            if not repository.has_external_sensor(self._selector.sensor_name):
+                raise UserFacingGraphQLError(
+                    GrapheneSensorNotFoundError(self._selector.sensor_name)
+                )
+            sensor_data: Union[SensorExecutionData, SerializableErrorInfo]
+            try:
+                sensor_data = repository_location.get_external_sensor_execution_data(
+                    name=self._selector.sensor_name,
+                    instance=graphene_info.context.instance,
+                    repository_handle=repository.handle,
+                    cursor=self._cursor,
+                    last_completion_time=None,
+                    last_run_key=None,
+                )
+            except Exception:
+                sensor_data = serializable_error_info_from_exc_info(sys.exc_info())
+            return GrapheneTickEvaluation(sensor_data)
+        else:
+            if not repository.has_external_schedule(self._selector.schedule_name):
+                raise UserFacingGraphQLError(
+                    GrapheneScheduleNotFoundError(self._selector.schedule_name)
+                )
+            if not self.timestamp:
+                raise Exception(
+                    "No tick timestamp provided when attempting to dry-run schedule"
+                    f" {self._selector.schedule_name}."
+                )
+            external_schedule = repository.get_external_schedule(self._selector.schedule_name)
+            timezone_str = external_schedule.execution_timezone
+            if not timezone_str:
+                timezone_str = "UTC"
+
+            next_tick_datetime = next(external_schedule.execution_time_iterator(self.timestamp))
+            schedule_time = to_timezone(pendulum.instance(next_tick_datetime), timezone_str)
+            schedule_data: Union[ScheduleExecutionData, SerializableErrorInfo]
+            try:
+                schedule_data = repository_location.get_external_schedule_execution_data(
+                    instance=graphene_info.context.instance,
+                    repository_handle=repository.handle,
+                    schedule_name=external_schedule.name,
+                    scheduled_execution_time=schedule_time,
+                )
+            except Exception:
+                schedule_data = serializable_error_info_from_exc_info(sys.exc_info())
+
+            return GrapheneTickEvaluation(schedule_data)
 
 
 class GrapheneTickEvaluation(graphene.ObjectType):
-    runRequests = graphene.List(lambda: GrapheneRunRequest)
+    runRequests = graphene.List(lambda: graphene.NonNull(GrapheneRunRequest))
     skipReason = graphene.String()
     error = graphene.Field(GraphenePythonError)
+    cursor = graphene.String()
 
     class Meta:
         name = "TickEvaluation"
 
-    def __init__(self, schedule_data):
+    def __init__(self, execution_data):
         check.inst_param(
-            schedule_data,
-            "schedule_data",
-            (ScheduleExecutionData, SerializableErrorInfo),
+            execution_data,
+            "execution_data",
+            (ScheduleExecutionData, SensorExecutionData, SerializableErrorInfo),
         )
         error = (
-            GraphenePythonError(schedule_data)
-            if isinstance(schedule_data, SerializableErrorInfo)
+            GraphenePythonError(execution_data)
+            if isinstance(execution_data, SerializableErrorInfo)
             else None
         )
         skip_reason = (
-            schedule_data.skip_message if isinstance(schedule_data, ScheduleExecutionData) else None
+            execution_data.skip_message
+            if not isinstance(execution_data, SerializableErrorInfo)
+            else None
         )
         self._run_requests = (
-            schedule_data.run_requests if isinstance(schedule_data, ScheduleExecutionData) else None
+            execution_data.run_requests
+            if not isinstance(execution_data, SerializableErrorInfo)
+            else None
         )
-        super().__init__(skipReason=skip_reason, error=error)
+        cursor = execution_data.cursor if isinstance(execution_data, SensorExecutionData) else None
+        super().__init__(skipReason=skip_reason, error=error, cursor=cursor)
 
     def resolve_runRequests(self, _graphene_info: ResolveInfo):
         if not self._run_requests:
@@ -281,12 +328,12 @@ class GrapheneRunRequest(graphene.ObjectType):
         return dump_run_config_yaml(self._run_request.run_config)
 
 
-class GrapheneFutureInstigationTicks(graphene.ObjectType):
-    results = non_null_list(GrapheneFutureInstigationTick)
+class GrapheneDryRunInstigationTicks(graphene.ObjectType):
+    results = non_null_list(GrapheneDryRunInstigationTick)
     cursor = graphene.NonNull(graphene.Float)
 
     class Meta:
-        name = "FutureInstigationTicks"
+        name = "DryRunInstigationTicks"
 
 
 class GrapheneInstigationState(graphene.ObjectType):
@@ -313,7 +360,7 @@ class GrapheneInstigationState(graphene.ObjectType):
         cursor=graphene.String(),
         statuses=graphene.List(graphene.NonNull(GrapheneInstigationTickStatus)),
     )
-    nextTick = graphene.Field(GrapheneFutureInstigationTick)
+    nextTick = graphene.Field(GrapheneDryRunInstigationTick)
     runningCount = graphene.NonNull(graphene.Int)  # remove with cron scheduler
 
     class Meta:
@@ -367,11 +414,10 @@ class GrapheneInstigationState(graphene.ObjectType):
 
         return None
 
-    def resolve_runs(self, graphene_info: ResolveInfo, **kwargs):
+    def resolve_runs(self, graphene_info: ResolveInfo, limit: Optional[int] = None):
         from .pipelines.pipeline import GrapheneRun
 
-        if kwargs.get("limit") and self._batch_loader:
-            limit = kwargs["limit"]
+        if limit and self._batch_loader:
             records = (
                 self._batch_loader.get_run_records_for_sensor(self._instigator_state.name, limit)
                 if self._instigator_state.instigator_type == InstigatorType.SENSOR
@@ -400,7 +446,7 @@ class GrapheneInstigationState(graphene.ObjectType):
             GrapheneRun(record)
             for record in graphene_info.context.instance.get_run_records(
                 filters=filters,
-                limit=kwargs.get("limit"),
+                limit=limit,
             )
         ]
 
@@ -519,8 +565,8 @@ class GrapheneInstigationStatesOrError(graphene.Union):
 
 
 types = [
-    GrapheneFutureInstigationTick,
-    GrapheneFutureInstigationTicks,
+    GrapheneDryRunInstigationTick,
+    GrapheneDryRunInstigationTicks,
     GrapheneInstigationTypeSpecificData,
     GrapheneInstigationState,
     GrapheneInstigationStateNotFoundError,
