@@ -2,11 +2,13 @@ from typing import TYPE_CHECKING, Optional, Union
 
 import dagster._check as check
 import graphene
-from dagster._core.definitions.events import AssetLineageInfo
-from dagster._core.events import DagsterEventType
+from dagster._core.events import AssetLineageInfo, DagsterEventType
+from dagster._core.events.log import EventLogEntry
 from dagster._core.execution.plan.objects import ErrorSource
 from dagster._core.execution.stats import RunStepKeyStatsSnapshot
 
+from dagster_graphql.schema.tags import GrapheneEventTag
+
 from ...implementation.events import construct_basic_params
 from ...implementation.fetch_runs import get_run_by_id, get_step_stats
 from ...implementation.loader import BatchRunLoader
@@ -314,6 +316,7 @@ class AssetEventMixin:
     runOrError = graphene.NonNull("dagster_graphql.schema.pipelines.pipeline.GrapheneRunOrError")
     stepStats = graphene.NonNull(lambda: GrapheneRunStepStats)
     partition = graphene.Field(graphene.String)
+    tags = non_null_list(GrapheneEventTag)
 
     def __init__(self, event, metadata):
         self._event = event
@@ -347,6 +350,14 @@ class AssetEventMixin:
     def resolve_partition(self, _graphene_info: ResolveInfo):
         return self._metadata.partition
 
+    def resolve_tags(self, _graphene_info):
+        if self._metadata.tags is None:
+            return []
+        else:
+            return [
+                GrapheneEventTag(key=key, value=value) for key, value in self._metadata.tags.items()
+            ]
+
 
 class GrapheneMaterializationEvent(graphene.ObjectType, AssetEventMixin):
     class Meta:
@@ -355,11 +366,12 @@ class GrapheneMaterializationEvent(graphene.ObjectType, AssetEventMixin):
 
     assetLineage = non_null_list(GrapheneAssetLineageInfo)
 
-    def __init__(self, event, assetLineage=None, loader=None):
+    def __init__(self, event: EventLogEntry, assetLineage=None, loader=None):
         self._asset_lineage = check.opt_list_param(assetLineage, "assetLineage", AssetLineageInfo)
         self._batch_run_loader = check.opt_inst_param(loader, "loader", BatchRunLoader)
 
-        materialization = event.dagster_event.step_materialization_data.materialization
+        dagster_event = check.not_none(event.dagster_event)
+        materialization = dagster_event.step_materialization_data.materialization
         super().__init__(**_construct_asset_event_metadata_params(event, materialization))
         AssetEventMixin.__init__(
             self,
