@@ -1,3 +1,5 @@
+from typing import Any, Optional
+
 import dagster._check as check
 import graphene
 from dagster._core.host_representation import RepresentedPipeline
@@ -32,7 +34,7 @@ class GrapheneRunConfigSchema(graphene.ObjectType):
 
     isRunConfigValid = graphene.Field(
         graphene.NonNull(GraphenePipelineConfigValidationResult),
-        args={"runConfigData": graphene.Argument(GrapheneRunConfigData)},
+        runConfigData=graphene.Argument(GrapheneRunConfigData),
         description="""Parse a particular run config result. The return value
         either indicates that the validation succeeded by returning
         `PipelineConfigValidationValid` or that there are configuration errors
@@ -74,12 +76,16 @@ class GrapheneRunConfigSchema(graphene.ObjectType):
             self._represented_pipeline.get_mode_def_snap(self._mode).root_config_key,
         )
 
-    def resolve_isRunConfigValid(self, graphene_info: ResolveInfo, **kwargs):
+    def resolve_isRunConfigValid(
+        self,
+        graphene_info: ResolveInfo,
+        runConfigData: Optional[Any] = None,  # custom scalar (GrapheneRunConfigData)
+    ):
         return resolve_is_run_config_valid(
             graphene_info,
             self._represented_pipeline,
             self._mode,
-            parse_run_config_input(kwargs.get("runConfigData", {}), raise_on_error=False),
+            parse_run_config_input(runConfigData or {}, raise_on_error=False),
         )
 
 
